/*
Module : IPenum.cpp
Purpose: Implementation for a class wrapper to get the details of all network adapters installed on your computer
Created: PJN / 21-04-1998
History: PJN / 04-01-1999 Removed win32sup.h include
         PJN / 27-12-2016 1. Minor update to the documentation.
                          2. Provision of a VC 5 workspace file now as standard
                          3. UNICODE enabled all the code.
                          4. IPENUM now uses MFC DLL and C Runtime DLL's instead of statically linking. This means 
                          that the IPENUM executable requires these files to be installed. If you haven't already 
                          got the MFC 4.2b DLLs on your machine (MFC42.DLL in your Windows system directory), 
                          then grab them from Microsoft's web site at http://activex.microsoft.com/controls/vc/mfc42.cab.
         PJN / 27-12-2016 1. Updated copyright details
                          2. Replaced all occurrences of NULL with nullptr throughout the codebase
                          3. Updated the code to clean compile on VC 2010 to VC 2015. The code now requires VC 2010 at a 
                          minimum.
                          4. Reworked the code to use the GetAdaptersAddresses API.
                          5. Added SAL annotations to all the code
         PJN / 27-12-2017 1. Updated copyright details.
                          2. Replaced CString::operator LPC*STR() calls throughout the codebase with CString::GetString 
                          calls
         PJN / 17-10-2018 1. Updated copyright details.
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean that the code will
                          now only compile on VC 2017 or later.
         PJN / 03-06-2019 1. Updated copyright details.
                          2. Updated the code to clean compile on VC 2019
         PJN / 09-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.
         PJN / 04-05-2022 1. Updated copyright details.
                          2. Updated the code to use C++ uniform initialization for all variable declarations.

Copyright (c) 1998 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "IPenum.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#pragma comment(lib, "Iphlpapi.lib")


//////////////////// Implementation ///////////////////////////////////////////

PIP_ADAPTER_ADDRESSES CIPEnum::Enumerate(_In_ ULONG Family, _In_ ULONG Flags, _In_ DWORD dwMaxTries)
{
  ULONG dwBuf{15000};
  bool bSuccess{false};
  DWORD dwIterations{0};
  DWORD dwError{ERROR_SUCCESS};
  while (!bSuccess && (dwIterations < dwMaxTries))
  {
    //Allocate some heap memory
    m_Addresses.resize(dwBuf);

    //Call the API
#pragma warning(suppress: 26490)
    dwError = GetAdaptersAddresses(Family, Flags, nullptr, reinterpret_cast<PIP_ADAPTER_ADDRESSES>(m_Addresses.data()), &dwBuf);
    if (dwError == ERROR_BUFFER_OVERFLOW)
    {
      m_Addresses.clear();
      SetLastError(dwError);
    }
    else
      bSuccess = true;

    ++dwIterations;
  }

#pragma warning(suppress: 26490)
  return reinterpret_cast<PIP_ADAPTER_ADDRESSES>(m_Addresses.data());
}
