#include "stdafx.h"
#include "IPenum.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


class CMyCommandLineInfo : public CCommandLineInfo
{
public:
//Constructors / Destructors
  CMyCommandLineInfo() noexcept : m_bIPv4{false},
                                  m_bIPv6{false},
                                  m_bHelp{false}
  {
  }
  CMyCommandLineInfo(const CMyCommandLineInfo&) = delete;
  CMyCommandLineInfo(CMyCommandLineInfo&&) = delete;
  ~CMyCommandLineInfo() = default; //NOLINT(modernize-use-override)

//Methods
  CMyCommandLineInfo& operator=(const CMyCommandLineInfo&) = delete;
  CMyCommandLineInfo& operator=(CMyCommandLineInfo&&) = delete;
  void ParseParam(const TCHAR* pszParam, BOOL bFlag, BOOL bLast) override
  {
    if (bFlag)
    {
      if ((_tcsicmp(pszParam, _T("h")) == 0) || (_tcsicmp(pszParam, _T("?")) == 0))
        m_bHelp = true;
      else if (_tcsicmp(pszParam, _T("4")) == 0)
        m_bIPv4 = true;
      else if (_tcsicmp(pszParam, _T("6")) == 0)
        m_bIPv6 = true;
      else
        __super::ParseParam(pszParam, bFlag, bLast);
    }
    else
      __super::ParseParam(pszParam, bFlag, bLast);
  }

  //Member variables
  bool m_bIPv4;
  bool m_bIPv6;
  bool m_bHelp;
};


//The one and only MFC app instance
#pragma warning(suppress: 26426)
CWinApp theApp;


CString AddressToString(_In_reads_bytes_(nSockAddrLen) const SOCKADDR* pSockAddr, _In_ int nSockAddrLen, _In_ int nFlags, _Inout_opt_ UINT* pnSocketPort = nullptr)
{
  //What will be the return value from this function
  CString sSocketAddress;
  CString sName;
#ifdef _UNICODE
  const int nResult{GetNameInfoW(pSockAddr, nSockAddrLen, sName.GetBuffer(NI_MAXHOST), NI_MAXHOST, nullptr, 0, nFlags)};
#else
  const int nResult{getnameinfo(pSockAddr, nSockAddrLen, sName.GetBuffer(NI_MAXHOST), NI_MAXHOST, nullptr, 0, nFlags)};
#endif //#ifdef _UNICODE
  sName.ReleaseBuffer();
  if (nResult == 0)
  {
    sSocketAddress = sName;
    if (pnSocketPort != nullptr)
      *pnSocketPort = ntohs(SS_PORT(&pSockAddr));
  }

  return sSocketAddress;
}

void PrintSocketAddress(_In_ const SOCKET_ADDRESS* pAddress)
{
#pragma warning(suppress: 26486)
  _tprintf(_T(" %s\n"), AddressToString(pAddress->lpSockaddr, pAddress->iSockaddrLength, NI_NUMERICHOST).GetString());
}

void PrintUnicastAddress(_In_ const IP_ADAPTER_UNICAST_ADDRESS* pAddress)
{
  PrintSocketAddress(&pAddress->Address);
  if (pAddress->Flags & IP_ADAPTER_ADDRESS_DNS_ELIGIBLE)
    _tprintf(_T("  IP Address is legal to appear in DNS\n"));
  if (pAddress->Flags & IP_ADAPTER_ADDRESS_TRANSIENT)
    _tprintf(_T("  IP Address is a cluster address\n"));
  switch (pAddress->PrefixOrigin)
  {
    case IpPrefixOriginOther:
    {
      _tprintf(_T("  IP prefix was provided by an other source\n"));
      break;
    }
    case IpPrefixOriginManual:
    {
      _tprintf(_T("  IP prefix was manually specified\n"));
      break;
    }
    case IpPrefixOriginDhcp:
    {
      _tprintf(_T("  IP prefix was provided by DHCP\n"));
      break;
    }
    case IpPrefixOriginRouterAdvertisement:
    {
      _tprintf(_T("  IP prefix was obtained through a router advertisement\n"));
      break;
    }
    case IpPrefixOriginUnchanged:
    {
      _tprintf(_T("  IP prefix should be unchanged\n"));
      break;
    }
    default:
    {
      _tprintf(_T("  IP prefix: %d\n"), pAddress->PrefixOrigin);
    }
  }
  switch (pAddress->SuffixOrigin)
  {
    case IpSuffixOriginOther:
    {
      _tprintf(_T("  IP suffix was provided by an other source\n"));
      break;
    }
    case IpSuffixOriginManual:
    {
      _tprintf(_T("  IP suffix was manually specified\n"));
      break;
    }
    case IpSuffixOriginWellKnown:
    {
      _tprintf(_T("  IP suffix was provided by well-known source\n"));
      break;
    }
    case IpSuffixOriginDhcp:
    {
      _tprintf(_T("  IP suffix was provided by DHCP\n"));
      break;
    }
    case IpSuffixOriginLinkLayerAddress:
    {
      _tprintf(_T("  IP suffix was provided by link-layer address\n"));
      break;
    }
    case IpSuffixOriginRandom:
    {
      _tprintf(_T("  IP suffix was obtained from a random source\n"));
      break;
    }
    case IpSuffixOriginUnchanged:
    {
      _tprintf(_T("  IP suffix should be unchanged\n"));
      break;
    }
    default:
    {
      _tprintf(_T("  IP suffix: %d"), pAddress->PrefixOrigin);
    }
  }
  switch (pAddress->DadState)
  {
    case IpDadStateInvalid:
    {
      _tprintf(_T("  DAD state is invalid\n"));
      break;
    }
    case IpDadStateTentative:
    {
      _tprintf(_T("  DAD state is tentative\n"));
      break;
    }
    case IpDadStateDuplicate:
    {
      _tprintf(_T("  DAD state is duplicate\n"));
      break;
    }
    case IpDadStateDeprecated:
    {
      _tprintf(_T("  DAD state is deprecated\n"));
      break;
    }
    case IpDadStatePreferred:
    {
      _tprintf(_T("  DAD state is preferred\n"));
      break;
    }
    default:
    {
      _tprintf(_T("  DAD state: %d"), pAddress->DadState);
    }
  }
  if (pAddress->ValidLifetime == 0xFFFFFFFF)
    _tprintf(_T("  Valid Lifetime: Infinite\n"));
  else
    _tprintf(_T("  Valid Lifetime: %u seconds\n"), pAddress->ValidLifetime); //NOLINT(clang-diagnostic-format)
  if (pAddress->PreferredLifetime == 0xFFFFFFFF)
    _tprintf(_T("  Preferred Lifetime: Infinite\n"));
  else
    _tprintf(_T("  Preferred Lifetime: %u seconds\n"), pAddress->PreferredLifetime); //NOLINT(clang-diagnostic-format)
  _tprintf(_T("  Lease lifetime: %u\n"), pAddress->LeaseLifetime); //NOLINT(clang-diagnostic-format)
#pragma warning(suppress: 26472)
  _tprintf(_T("  Prefix or Network part length of address: %d\n"), static_cast<int>(pAddress->OnLinkPrefixLength));
}

void PrintAnycastAddress(_In_ const IP_ADAPTER_ANYCAST_ADDRESS* pAddress)
{
  PrintSocketAddress(&pAddress->Address);
  if (pAddress->Flags & IP_ADAPTER_ADDRESS_DNS_ELIGIBLE)
    _tprintf(_T("  IP Address is legal to appear in DNS\n"));
  if (pAddress->Flags & IP_ADAPTER_ADDRESS_TRANSIENT)
    _tprintf(_T("  IP Address is a cluster address\n"));
}

void PrintMulticastAddress(_In_ const IP_ADAPTER_MULTICAST_ADDRESS* pAddress)
{
  PrintSocketAddress(&pAddress->Address);
  if (pAddress->Flags & IP_ADAPTER_ADDRESS_DNS_ELIGIBLE)
    _tprintf(_T("  IP Address is legal to appear in DNS\n"));
  if (pAddress->Flags & IP_ADAPTER_ADDRESS_TRANSIENT)
    _tprintf(_T("  IP Address is a cluster address\n"));
}

void PrintDnsServerAddress(_In_ const IP_ADAPTER_DNS_SERVER_ADDRESS* pAddress)
{
  PrintSocketAddress(&pAddress->Address);
}

void PrintWinsServerAddress(_In_ const IP_ADAPTER_WINS_SERVER_ADDRESS* pAddress)
{
  PrintSocketAddress(&pAddress->Address);
}

void PrintGatewayAddress(_In_ const IP_ADAPTER_GATEWAY_ADDRESS* pAddress)
{
  PrintSocketAddress(&pAddress->Address);
}

void PrintPrefix(_In_ const IP_ADAPTER_PREFIX* pPrefix)
{
  PrintSocketAddress(&pPrefix->Address);
  _tprintf(_T("  Prefix length: %u\n"), pPrefix->PrefixLength); //NOLINT(clang-diagnostic-format)
}

#pragma warning(suppress: 26485)
int _tmain(int /*argc*/, TCHAR* /*argv*/[], TCHAR* /*envp*/[]) //NOLINT(modernize-avoid-c-arrays)
{
  //Initialize MFC
  if (!AfxWinInit(::GetModuleHandle(nullptr), nullptr, ::GetCommandLine(), 0))
  {
    _tprintf(_T("Failed to initialize MFC\n"));
    return 1;
  }

  //Initialize Winsock v2.0
  WSADATA wsaData{};
  if (WSAStartup(MAKEWORD(2, 0), &wsaData) != 0)
  {
    _tprintf(_T("Failed to initialize Winsock\n"));
    return 1;
  }

  //Confirm that the WinSock DLL supports 2.0 
  if (LOBYTE(wsaData.wVersion) != 2)
  {
    _tprintf(_T("Installed Winsock version does not support v2.0\n"));
    WSACleanup();
  }


  //Parse the command line
  CMyCommandLineInfo cmdInfo;
  theApp.ParseCommandLine(cmdInfo);

  //Display help if necessary
  if (cmdInfo.m_bHelp)
  {
    _tprintf(_T("IPEnum [/h] [/?] [/4 [6]n"));
    _tprintf(_T("/h or /? displays this help\n"));
    _tprintf(_T("/4 displays IPv4 addresses\n"));
    _tprintf(_T("/6 displays IPv6 addresses\n"));
    _tprintf(_T(" if neither /4 or /6 is specified then /4 and /6 is assumed\n"));
    WSACleanup();
    return 0;
  }

  //Do the enumeration
  CIPEnum ipenum;
  ULONG nFamily{AF_UNSPEC};
  if (cmdInfo.m_bIPv4 && !cmdInfo.m_bIPv6)
    nFamily = AF_INET;
  else if (cmdInfo.m_bIPv6 && !cmdInfo.m_bIPv4)
    nFamily = AF_INET6;
  PIP_ADAPTER_ADDRESSES pAddresses{ipenum.Enumerate(nFamily, GAA_FLAG_INCLUDE_WINS_INFO | GAA_FLAG_INCLUDE_GATEWAYS | GAA_FLAG_INCLUDE_ALL_INTERFACES)};

  //Display the results if successful
  if (pAddresses != nullptr)
  {
    PIP_ADAPTER_ADDRESSES pCurrAddress{pAddresses};
    while (pCurrAddress != nullptr)
    {
#pragma warning(suppress: 26489)
      _tprintf(_T("Friendly name: %wS\n"), pCurrAddress->FriendlyName);
#pragma warning(suppress: 26489)
      _tprintf(_T("Description: %wS\n"), pCurrAddress->Description);
#pragma warning(suppress: 26489)
      _tprintf(_T("DNS Suffix: %wS\n"), pCurrAddress->DnsSuffix);
#pragma warning(suppress: 26489)
      _tprintf(_T("Adapter name: %hs\n"), pCurrAddress->AdapterName);
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_DDNS_ENABLED)
        _tprintf(_T("Dynamic DNS is enabled\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_REGISTER_ADAPTER_SUFFIX)
        _tprintf(_T("Register the DNS suffix\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_DHCP_ENABLED)
        _tprintf(_T("DHCP is enabled\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_RECEIVE_ONLY)
        _tprintf(_T("Adapter is a receive-only adapter\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_NO_MULTICAST)
        _tprintf(_T("Not a multicast recipient\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_IPV6_OTHER_STATEFUL_CONFIG)
        _tprintf(_T("Contains other IPv6 specific stateful configuration information\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_NETBIOS_OVER_TCPIP_ENABLED)
        _tprintf(_T("NetBIOS over TCP/IP is enabled\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_IPV4_ENABLED)
        _tprintf(_T("IPv4 is enabled\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_IPV6_ENABLED)
        _tprintf(_T("IPv6 is enabled\n"));
#pragma warning(suppress: 26489)
      if (pCurrAddress->Flags & IP_ADAPTER_IPV6_MANAGE_ADDRESS_CONFIG)
        _tprintf(_T("IPv6 managed address configuration is enabled\n"));
#pragma warning(suppress: 26489)
      _tprintf(_T("MTU: %u bytes\n"), pCurrAddress->Mtu); //NOLINT(clang-diagnostic-format)
      switch (pCurrAddress->IfType)
      {
        case IF_TYPE_OTHER:
        {
          _tprintf(_T("Interface Type: Some other type of network interface\n"));
          break;
        }
        case IF_TYPE_ETHERNET_CSMACD:
        {
          _tprintf(_T("Interface Type: Ethernet\n"));
          break;
        }
        case IF_TYPE_ISO88025_TOKENRING:
        {
          _tprintf(_T("Interface Type: Token ring\n"));
          break;
        }
        case IF_TYPE_PPP:
        {
          _tprintf(_T("Interface Type: PPP\n"));
          break;
        }
        case IF_TYPE_SOFTWARE_LOOPBACK:
        {
          _tprintf(_T("Interface Type: Software loopback\n"));
          break;
        }
        case IF_TYPE_ATM:
        {
          _tprintf(_T("Interface Type: ATM\n"));
          break;
        }
        case IF_TYPE_IEEE80211:
        {
          _tprintf(_T("Interface Type: IEEE 802.11 Wireless\n"));
          break;
        }
        case IF_TYPE_TUNNEL:
        {
          _tprintf(_T("Interface Type: Tunnel\n"));
          break;
        }
        case IF_TYPE_IEEE1394:
        {
          _tprintf(_T("Interface Type: IEEE 1394 (Firewire)\n"));
          break;
        }
        default:
        {
#pragma warning(suppress: 26489)
          _tprintf(_T("Interface Type: %u\n"), pCurrAddress->IfType); //NOLINT(clang-diagnostic-format)
          break;
        }
      }
      switch (pCurrAddress->OperStatus)
      {
        case IfOperStatusUp:
        {
          _tprintf(_T("Interface is up\n"));
          break;
        }
        case IfOperStatusDown:
        {
          _tprintf(_T("Interface is down\n"));
          break;
        }
        case IfOperStatusTesting:
        {
          _tprintf(_T("Interface is in testing mode\n"));
          break;
        }
        case IfOperStatusUnknown:
        {
          _tprintf(_T("Interface status is unknown\n"));
          break;
        }
        case IfOperStatusDormant:
        {
          _tprintf(_T("Interface is dormant\n"));
          break;
        }
        case IfOperStatusNotPresent:
        {
          _tprintf(_T("Interface is not present\n"));
          break;
        }
        case IfOperStatusLowerLayerDown:
        {
          _tprintf(_T("Interface is down because lower-layer interface is down\n"));
          break;
        }
        default:
        {
#pragma warning(suppress: 26489)
          _tprintf(_T("Status: %d\n"), pCurrAddress->OperStatus);
          break;
        }
      }
#pragma warning(suppress: 26489)
      _tprintf(_T("IfIndex (IPv4 interface): %u\n"), pCurrAddress->IfIndex); //NOLINT(clang-diagnostic-format)
#pragma warning(suppress: 26489)
      _tprintf(_T("Ipv6IfIndex (IPv6 interface): %u\n"), pCurrAddress->Ipv6IfIndex); //NOLINT(clang-diagnostic-format)
#pragma warning(suppress: 26489)
      _tprintf(_T("Ipv4Metric: %u\n"), pCurrAddress->Ipv4Metric); //NOLINT(clang-diagnostic-format)
      _tprintf(_T("Ipv6Metric: %u\n"), pCurrAddress->Ipv6Metric); //NOLINT(clang-diagnostic-format)
      if (pCurrAddress->TransmitLinkSpeed != 0xFFFFFFFFFFFFFFFF)
        _tprintf(_T("Transmit link speed: %I64u Mbit/s\n"), pCurrAddress->TransmitLinkSpeed / 1000000);
      if (pCurrAddress->ReceiveLinkSpeed != 0xFFFFFFFFFFFFFFFF)
        _tprintf(_T("Receive link speed: %I64u Mbit/s\n"), pCurrAddress->ReceiveLinkSpeed / 1000000);
      _tprintf(_T("Compartment ID: %u\n"), pCurrAddress->CompartmentId);
      LPOLESTR pszGUID{nullptr};
      if (SUCCEEDED(StringFromCLSID(pCurrAddress->NetworkGuid, &pszGUID)))
      {
        _tprintf(_T("Network Guid: %wS\n"), pszGUID);
        CoTaskMemFree(pszGUID);
      }
      switch (pCurrAddress->ConnectionType)
      {
        case NET_IF_CONNECTION_DEDICATED:
        {
          _tprintf(_T("Connection type is dedicated\n"));
          break;
        }
        case NET_IF_CONNECTION_PASSIVE:
        {
          _tprintf(_T("Connection type is passive\n"));
          break;
        }
        case NET_IF_CONNECTION_DEMAND:
        {
          _tprintf(_T("Connection type is dedicated\n"));
          break;
        }
        default:
        {
          _tprintf(_T("Connection type: %d\n"), pCurrAddress->ConnectionType);
          break;
        }
      }
      switch (pCurrAddress->TunnelType)
      {
        case TUNNEL_TYPE_NONE:
        {
          _tprintf(_T("Not a tunnel\n"));
          break;
        }
        case TUNNEL_TYPE_OTHER:
        {
          _tprintf(_T("Other tunnel type\n"));
          break;
        }
        case TUNNEL_TYPE_DIRECT:
        {
          _tprintf(_T("Tunnel type: Direct\n"));
          break;
        }
        case TUNNEL_TYPE_6TO4:
        {
          _tprintf(_T("Tunnel type: 6To4\n"));
          break;
        }
        case TUNNEL_TYPE_ISATAP:
        {
          _tprintf(_T("Tunnel type: ISATAP\n"));
          break;
        }
        case TUNNEL_TYPE_TEREDO:
        {
          _tprintf(_T("Tunnel type: Teredo\n"));
          break;
        }
        case TUNNEL_TYPE_IPHTTPS:
        {
          _tprintf(_T("Tunnel type: HTTPS\n"));
          break;
        }
        default:
        {
          _tprintf(_T("Connection type: %d\n"), pCurrAddress->ConnectionType);
          break;
        }
      }
      if (pCurrAddress->Dhcpv4Server.iSockaddrLength)
        _tprintf(_T("DHCPv4 server: %s\n"), AddressToString(pCurrAddress->Dhcpv4Server.lpSockaddr, pCurrAddress->Dhcpv4Server.iSockaddrLength, NI_NUMERICHOST).GetString());

      //Display the physical address
      if (pCurrAddress->PhysicalAddressLength != 0)
      {
        _tprintf(_T("Physical address: "));
        for (DWORD i{0}; i<pCurrAddress->PhysicalAddressLength; i++)
        {
          if (i == (pCurrAddress->PhysicalAddressLength - 1))
#pragma warning(suppress: 26446 26472 26482)
            _tprintf(_T("%.2X\n"), static_cast<int>(pCurrAddress->PhysicalAddress[i]));
          else
#pragma warning(suppress: 26446 26472 26482)
            _tprintf(_T("%.2X-"), static_cast<int>(pCurrAddress->PhysicalAddress[i]));
        }
      }

      _tprintf(_T("DHCPv6 DUID: "));
      for (DWORD i{0}; i<sizeof(pCurrAddress->Dhcpv6ClientDuid); i++)
      {
        if (i == (sizeof(pCurrAddress->Dhcpv6ClientDuid) - 1))
#pragma warning(suppress: 26446 26472 26482)
          _tprintf(_T("%.2X\n"), static_cast<int>(pCurrAddress->Dhcpv6ClientDuid[i]));
        else
#pragma warning(suppress: 26446 26472 26482)
          _tprintf(_T("%.2X-"), static_cast<int>(pCurrAddress->Dhcpv6ClientDuid[i]));
      }
      _tprintf(_T("DHCPv6 LAID: %u\n"), pCurrAddress->Dhcpv6Iaid); //NOLINT(clang-diagnostic-format)

      //Display the unicast addresses
#pragma warning(suppress: 26489)
      PIP_ADAPTER_UNICAST_ADDRESS pUnicast{pCurrAddress->FirstUnicastAddress};
      if (pUnicast != nullptr)
      {
        _tprintf(_T("Unicast addresses\n"));
        for (; pUnicast != nullptr; pUnicast = pUnicast->Next)
#pragma warning(suppress: 26486)
          PrintUnicastAddress(pUnicast);
      }

      //Display the anycast addresses
#pragma warning(suppress: 26489)
      PIP_ADAPTER_ANYCAST_ADDRESS pAnycast{pCurrAddress->FirstAnycastAddress};
      if (pAnycast != nullptr)
      {
        _tprintf(_T("Anycast addresses\n"));
        for (; pAnycast != nullptr; pAnycast = pAnycast->Next)
#pragma warning(suppress: 26486)
          PrintAnycastAddress(pAnycast);
      }

      //Display the multicast addresses
#pragma warning(suppress: 26489)
      PIP_ADAPTER_MULTICAST_ADDRESS pMulticast{pCurrAddress->FirstMulticastAddress};
      if (pMulticast != nullptr)
      {
        _tprintf(_T("Multicast addresses\n"));
        for (; pMulticast != nullptr; pMulticast = pMulticast->Next)
#pragma warning(suppress: 26486)
          PrintMulticastAddress(pMulticast);
      }

      //Display the DNS server addresses
#pragma warning(suppress: 26489)
      IP_ADAPTER_DNS_SERVER_ADDRESS* pDnsServer{pCurrAddress->FirstDnsServerAddress};
      if (pDnsServer != nullptr)
      {
        _tprintf(_T("DNS server addresses\n"));
        for (; pDnsServer != nullptr; pDnsServer = pDnsServer->Next)
#pragma warning(suppress: 26486)
          PrintDnsServerAddress(pDnsServer);
      }

      //Display the WINS server addresses
#pragma warning(suppress: 26489)
      IP_ADAPTER_WINS_SERVER_ADDRESS* pWinsServer{pCurrAddress->FirstWinsServerAddress};
      if (pWinsServer != nullptr)
      {
        _tprintf(_T("WINS server addresses\n"));
        for (; pWinsServer != nullptr; pWinsServer = pWinsServer->Next)
#pragma warning(suppress: 26486)
          PrintWinsServerAddress(pWinsServer);
      }

      //Display the gateway addresses
#pragma warning(suppress: 26489)
      IP_ADAPTER_GATEWAY_ADDRESS* pGateway{pCurrAddress->FirstGatewayAddress};
      if (pGateway != nullptr)
      {
        _tprintf(_T("Gateway server addresses\n"));
        for (; pGateway != nullptr; pGateway = pGateway->Next)
#pragma warning(suppress: 26486)
          PrintGatewayAddress(pGateway);
      }

      //Display the DNS suffixes
#pragma warning(suppress: 26489)
      IP_ADAPTER_DNS_SUFFIX* pDnsSuffix{pCurrAddress->FirstDnsSuffix};
      if (pGateway != nullptr)
      {
        _tprintf(_T("DNS Suffixes\n"));
#pragma warning(suppress: 26489)
        for (; pDnsSuffix != nullptr; pDnsSuffix = pDnsSuffix->Next)
#pragma warning(suppress: 26485 26489)
          _tprintf(_T(" DNS Suffix: %wS\n"), pDnsSuffix->String);
      }

      //Display Zone indices data
      _tprintf(_T("ZoneIndices (hex): "));
      for (const auto ZoneIndice : pCurrAddress->ZoneIndices)
        _tprintf(_T("%lx "), ZoneIndice);
      _tprintf(_T("\n"));

      //Display the prefix data
#pragma warning(suppress: 26489)
      IP_ADAPTER_PREFIX* pPrefix{pCurrAddress->FirstPrefix};
      if (pPrefix != nullptr)
      {
        _tprintf(_T("Prefixes\n"));
        for (; pPrefix != nullptr; pPrefix = pPrefix->Next)
#pragma warning(suppress: 26486)
          PrintPrefix(pPrefix);
      }

      //Prepare for the next loop
      _tprintf(_T("\n"));
#pragma warning(suppress: 26489)
      pCurrAddress = pCurrAddress->Next;
    }
  }
  else
    _tprintf(_T("Failed to enumerate IP addresses, Error:%u\n"), GetLastError()); //NOLINT(clang-diagnostic-format)

  WSACleanup();
  return 0;
}
